import io
import logging

from taichu_storage import StorageInterface
from minio import Minio
from io import BytesIO


class StorageMinio(StorageInterface):
    _use_alluxio_path = True

    def __init__(self, cfgs=None):
        if cfgs is None:
            cfgs = {}

        endpoint = cfgs.get('minio_endpoint')
        ak = cfgs.get('minio_ak')
        sk = cfgs.get('minio_sk')
        self._bucket = cfgs.get('bucket')
        self._use_alluxio_path = cfgs.get('use_alluxio_path', True)
        print(self._use_alluxio_path, self._bucket)

        self._client = Minio(
            endpoint,
            ak,
            sk,
            secure=False,
        )

    def _prefix_key(self, key):
        alluxio_path = 'data/%s/' % self._bucket
        print('alluxio_path: %s' % alluxio_path)
        if self._use_alluxio_path is False or key.startswith(alluxio_path):
            k = key
        else:
            k = alluxio_path + key.lstrip('/')
        print(k)
        return k

    def _alluxio_bucket(self):
        return 'alluxio' if self._use_alluxio_path else self._bucket

    def write_bytes(self, content_bytes, key):
        key = self._prefix_key(key)
        # o = StrReader(content_bytes)
        try:
            self._client.put_object(
                self._alluxio_bucket(),
                key,
                io.BytesIO(content_bytes),
                len(content_bytes)
            )
        except Exception as e:
            logging.error(e)

    def write_string(self, content_string, key):
        key = self._prefix_key(key)
        return self.write_bytes(content_string.encode('utf-8'), key)

    def upload_file(self, file_path, key):
        key = self._prefix_key(key)
        print(key)
        try:
            self._client.fput_object(
                self._alluxio_bucket(),
                key,
                file_path
            )
        except Exception as e:
            logging.error(e)


class StrReader:

    def __init__(self, strs):
        self.str = strs if isinstance(strs, bytes) else strs.encode('utf-8')

    def read(self, n=-1):
        return self.str

    def len(self):
        return len(self.str)


if __name__ == '__main__':
    c = StorageMinio({
        'bucket': 'publish-data',
        'minio_ak': 'minio',
        'minio_sk': 'minio2022',
        'minio_endpoint': '172.16.10.19:9000'
    })
    c.write_string('abc', 'sys/test/abc.txt')
    # c.write_bytes(b"hello", 'sys/test/abc_bytes.txt')
    c.upload_file('test/b.txt', 'sys/test/b.txt')
    # print(c.generate_signed_url('sys/test/abc.txt'))
    # print(c.generate_upload_credentials('sys/test/abc.txt'))



import base64
import io
import os
import json
from abc import abstractmethod
from enum import Enum


class Protocol(Enum):
    BOTO = 1
    OBS = 2
    BOTO3 = 3
    MINIO = 4


class StorageInterface:
    _client = None
    _bucket = None

    def client(self):
        return self._client

    @abstractmethod
    def write_bytes(self, content_bytes, key):
        pass

    @abstractmethod
    def write_string(self, content_string, key):
        pass

    def write_json(self, content_json, key):
        self.write_string(json.dumps(content_json), key)

    def write_base64(self, content_base64, key):
        image_data = base64.b64decode(content_base64)
        image_stream = io.BytesIO(image_data)
        self.write_bytes(image_stream, key)

    @abstractmethod
    def upload_file(self, file_path, key):
        pass

    def upload_directory(self, directory, key):
        for root, dirs, files in os.walk(directory):
            for file in files:
                local_path = os.path.join(root, file)
                s3_key = os.path.join(key, os.path.relpath(local_path, directory))
                self.upload_file(local_path, s3_key)

    @abstractmethod
    def download_file(self, file_path, key):
        raise Exception("Not Implemented: download_file")

    @abstractmethod
    def download_directory(self, key, local_target_directory):
        raise Exception("Not Implemented: download_directory")

    @abstractmethod
    def generate_signed_url(self, key, expiration=600, host_url=None):
        raise Exception("Not Implemented: generate_signed_url")

    @abstractmethod
    def generate_upload_credentials(self, key, expiration=3600):
        raise Exception("Not Implemented: generate_upload_credentials")

    # @abstractmethod
    # def copy_key(self, from_key, to_key, filter_func=None):
    #     raise Exception("Not Implemented: copy_key")
    #
    # @abstractmethod
    # def copy_dir(self, from_dir, to_dir, filter_func=None):
    #     raise Exception("Not Implemented: copy_dir")
    #
    # @abstractmethod
    # def list(self, dir, delimiter="", marker=None, size=None):
    #     raise Exception("Not Implemented: list")


def create_storage(protocol: Protocol, cfgs=None):
    if cfgs is None:
        cfgs = {}
    if protocol == Protocol.OBS:
        from taichu_storage.obs import StorageObs
        return StorageObs(cfgs)
    if protocol == Protocol.BOTO3:
        from taichu_storage.b3 import StorageBoto3
        return StorageBoto3(cfgs)
    else:
        from taichu_storage.boto_client import StorageAlluxio
        return StorageAlluxio()



import logging

import boto3
import botocore.exceptions
from botocore.config import Config

from taichu_storage import StorageInterface
from taichu_storage.env import *


class StorageBoto3(StorageInterface):
    def __init__(self, cfgs=None):
        if cfgs is None:
            cfgs = {}

        bucket = cfgs.get('boto3_bucket')
        ak = cfgs.get('boto3_ak')
        sk = cfgs.get('boto3_sk')
        region_name = cfgs.get('boto3_region_name')
        endpoint_url = cfgs.get('boto3_endpoint_url')

        self._bucket = bucket
        self._client = boto3.client(
            's3',
            aws_access_key_id=ak,
            aws_secret_access_key=sk,
            use_ssl=True,
            region_name=region_name,
            endpoint_url=endpoint_url,
            config=Config(s3={"addressing_style": "virtual", "signature_version": 's3v4'}))

    def write_bytes(self, content_bytes, key):
        self._client.put_object(Body=content_bytes, Bucket=self._bucket, Key=key)

    def write_string(self, content_string, key):
        try:
            data = content_string.encode('utf-8')
            self._client.put_object(Body=data, Bucket=self._bucket, Key=key)
        except Exception as e:
            logging.info("key: " + key)
            logging.error("TaichuStorageError", e)

    def upload_file(self, file_path, key):
        self._client.upload_file(file_path, self._bucket, key)

    def download_file(self, file_path, key):
        self._client.download_file(self._bucket, key, file_path)

    def download_directory(self, key, local_target_directory):
        response = self._client.list_objects_v2(Bucket=self._bucket, Prefix=key)
        m = {}
        for obj in response.get('Contents', []):
            s3_key = obj['Key']
            # local_file_path = os.path.join(local_target_directory, os.path.relpath(s3_key, key))
            local_file_path = f'{local_target_directory}{s3_key.replace(key, "")}'
            dirname = os.path.dirname(local_file_path)
            try:
                if not m.get(dirname, False):
                    os.makedirs(dirname, exist_ok=True)
                    m[dirname] = True
                    print('make dir: '+dirname)
                if os.path.isdir(local_file_path):
                    print(local_file_path + " is a directory continue")
                    continue
                self._client.download_file(self._bucket, s3_key, local_file_path)
                print(f'Downloaded s3://{self._bucket}/{s3_key} to {local_file_path}')
            except Exception as e:
                logging.error(e)
                logging.info(s3_key)
                logging.info(local_file_path)
                logging.info('dirname: %s' % dirname)

    def generate_signed_url(self, key, expiration=600, host_url=None):
        try:
            url = self._client.generate_presigned_url(
                'get_object',
                Params={'Bucket': self._bucket, 'Key': key},
                ExpiresIn=expiration
            )
            if not host_url:
                return url
            h = self._boto_host
            if self._boto_port is not 80:
                h = self._boto_host + ':' + str(self._boto_port)
            url = url.replace(h, host_url)
            print("URL_RETURN: ", url)
            return url
        except botocore.exceptions.ClientError as e:
            logging.error("Error generating presigned URL:", e)
            return None

    def generate_upload_credentials(self, key, expiration=3600):
        return self._client.generate_presigned_post(self._bucket, key, ExpiresIn=expiration)
        # {'url': 'http://192.168.0.149:39999/api/v1/s3/publish-data', 'fields': {'key': 'sys/test/abc.txt',
        # 'x-amz-algorithm': 'AWS4-HMAC-SHA256', 'x-amz-credential': 'root/20230704/cn/s3/aws4_request',
        # 'x-amz-date': '20230704T073038Z',
        # 'policy':
        # 'eyJleHBpcmF0aW9uIjogIjIwMjMtMDctMDRUMDg6MzA6MzhaIiwgImNvbmRpdGlvbnMiOiBbeyJidWNrZXQiOiAicHVibGlzaC1kYXRhIn0sIHsia2V5IjogInN5cy90ZXN0L2FiYy50eHQifSwgeyJ4LWFtei1hbGdvcml0aG0iOiAiQVdTNC1ITUFDLVNIQTI1NiJ9LCB7IngtYW16LWNyZWRlbnRpYWwiOiAicm9vdC8yMDIzMDcwNC9jbi9zMy9hd3M0X3JlcXVlc3QifSwgeyJ4LWFtei1kYXRlIjogIjIwMjMwNzA0VDA3MzAzOFoifV19',
        # 'x-amz-signature': 'accdd539815bb170132a109c1802630ab3bf1e582f7792e19d51100158cb5057'}}


if __name__ == '__main__':
    c = StorageBoto3({
        'bucket': 'publish-data',
        'boto3_ak': 'root',
        'boto3_sk': 'CBF5CCEC7425C19221F00D6A03B43B08',
        'boto3_region_name': 'cn',
        'boto3_endpoint_url': 'http://192.168.0.149:39999/api/v1/s3'
    })
    # c.write_string('abc', 'sys/test/abc.txt')
    # print(c.generate_signed_url('sys/test/abc.txt'))
    print(c.generate_upload_credentials('sys/test/abc.txt'))
    # c.write_json({'abc': "123"}, 'sys/test/json.json')
    # c.upload_file('test/b.txt', 'sys/test/b.txt')
    # c.upload_directory('test', 'sys/test/directory')
    # c.download_directory('sys/test/directory', 'test/download')




from setuptools import setup, find_packages

if __name__ == '__main__':
    name = 'taichu-storage'

    with open('requirements.txt', 'r') as f:
        requirements = f.read().splitlines()

    long_description = 'storage sdks aggregation'
    # with open('README.md', 'r') as f:
    #     long_description = f.read()

    setup(
        name=name,
        version='0.0.5',
        description='storage sdks aggregation',
        long_description=long_description,
        author='taichu platform team',
        python_requires=">=3.6.0",
        url='',
        keywords='taichu',
        packages=find_packages(),
        install_requires=requirements,
        include_package_data=True,
        package_data={
            '': ['*.sh'],
        }
    )



import os
import uuid

# 必填项目
STORAGE_TYPE = os.getenv('STORAGE_MEDIA', 'MINIO')

STORAGE_BUCKET = os.getenv('STORAGE_BUCKET', 'publish-data')
SERVICE_NAME = os.getenv('SERVICE_NAME', 'anonymous')

ALLUXIO_HOST = os.getenv('ALLUXIO_HOST', 'alluxio-proxy.infra')
ALLUXIO_PORT = int(os.getenv('ALLUXIO_PORT', 39999))
ALLUXIO_PATH = os.getenv('ALLUXIO_PATH', '/api/v1/s3')
S3_ACCESS_KEY_ID = os.getenv('S3_ACCESS_KEY_ID', 'Credential=root/')
S3_SECRET_ACCESS_KEY = os.getenv('S3_SECRET_ACCESS_KEY', '')

OBS_SERVER = os.getenv('OBS_SERVER')
OBS_ACCESS_KEY_ID = os.getenv('OBS_ACCESS_KEY_ID')
OBS_SECRET_ACCESS_KEY = os.getenv('OBS_SECRET_ACCESS_KEY')

service_name = None


def get_service_name():
    global service_name
    if service_name is None:
        env_sn = os.getenv('SERVICE_NAME', 'anonymous')
        if env_sn == 'anonymous':
            service_name = gen_random_name()
            return service_name
        else:
            service_name = env_sn
            return service_name
    else:
        return service_name


def gen_random_name():
    return str(uuid.uuid4()).replace('-', '')[:8]



import logging

from taichu_storage import StorageInterface
from taichu_storage.env import *
from obs import ObsClient, PutObjectHeader


class StorageObs(StorageInterface):

    def __init__(self, cfgs=None):
        if cfgs is None:
            cfgs = {}

        self._bucket = cfgs.get('obs_bucket')
        obs_ak = cfgs.get('obs_ak')
        obs_sk = cfgs.get('obs_sk')
        obs_server = cfgs.get('obs_server')

        self._client = ObsClient(
            access_key_id=obs_ak,
            secret_access_key=obs_sk,
            server=obs_server
        )

    def download_file(self, file_path, key):
        pass

    def download_directory(self, key, local_target_directory):
        pass

    def generate_signed_url(self, key, expiration=600, host_url=None):
        try:
            rps = self._client.createSignedUrl("GET", self._bucket, key, "", 6000, {}, {})
            return rps.signedUrl
        except Exception as e:
            logging.error(e)
            return ''

    def generate_upload_credentials(self, key, expiration=3600):
        pass

    def write_bytes(self, content_bytes, key):
        self.write_string(content_bytes, key)

    def write_string(self, content_string, key):
        try:
            self._client.putContent(self._bucket, key, content=content_string)
        except Exception as e:
            logging.info("key: " + key)
            logging.error("TaichuStorageError", e)

    def upload_file(self, file_path, key):
        headers = PutObjectHeader()
        headers.contentType = 'text/plain'
        self._client.putFile(self._bucket, key, file_path, metadata={}, headers=headers)



import logging

from taichu_storage import StorageInterface
import boto.s3.connection
from taichu_storage.env import *
import boto.exception


class StorageAlluxio(StorageInterface):
    _boto_host = ''
    _boto_port = ''

    def __init__(self, cfgs=None):
        if cfgs is None:
            cfgs = {}

        boto_ak = cfgs.get('boto_ak')
        boto_sk = cfgs.get('boto_sk')
        self._boto_host = cfgs.get('boto_host')
        self._boto_port = cfgs.get('boto_port')
        boto_path = cfgs.get('boto_path')
        boto_bucket = cfgs.get('boto_bucket')

        self._client = boto.connect_s3(
            aws_access_key_id=boto_ak,
            aws_secret_access_key=boto_sk,
            host=self._boto_host,
            port=self._boto_port,
            path=boto_path,
            is_secure=False,
            calling_format=boto.s3.connection.OrdinaryCallingFormat(),
        )
        self._bucket = self._client.get_bucket(boto_bucket)

    def write_bytes(self, content_bytes, key):
        try:
            s3_key = self._bucket.new_key(key)
            s3_key.set_contents_from_file(content_bytes)
        except boto.exception.BotoClientError:
            pass
        except Exception as e:
            logging.info("key: " + key)
            logging.error("TaichuStorageError", e)

    def write_string(self, content_string, key):
        try:
            s3_key = self._bucket.new_key(key)
            s3_key.set_contents_from_string(content_string)
        except boto.exception.BotoClientError:
            pass
        except Exception as e:
            logging.info("key: " + key)
            logging.error("TaichuStorageError", e)

    def upload_file(self, file_path, key):
        s3_key = self._bucket.new_key(key)
        with open(file_path, "rb") as f:
            try:
                s3_key.set_contents_from_file(f)
            except:
                return

    def download_dir(self, src, dest):
        rps = self._bucket.list(prefix=src)
        for r in rps:
            os.makedirs(dest, exist_ok=True)
            local_file = f'{dest}{r.name.replace(src, "")}'
            try:
                key = self._bucket.get_key(r.name)
                with open(local_file, 'wb') as f:
                    key.get_contents_to_file(f)
            except Exception as e:
                if 'SAXParseException' in str(type(e)):
                    pass
                else:
                    logging.error(e)

    def generate_signed_url(self, key, expiration=600, host_url=None):
        try:
            k = self._bucket.get_key(key)
            if k is None:
                logging.info(key, "：不存在")
                return key + "：不存在"
            url = k.generate_url(expiration)
            print("URL_ORIGIN: ", url)
            if not host_url:
                return url
            h = self._boto_host
            if self._boto_port is not 80:
                h = self._boto_host + ':' + str(self._boto_port)
            url = url.replace(h, host_url)
            print("URL_RETURN: ", url)
            return url
        except Exception as e:
            logging.error(e)
            return None
